/* arch/arm/mach-s3c2410/tomtomgo-devs.h
 *
 * Definitions of TomTom GO platform devices.
 *
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 * Author: Dimitry Andric <dimitry.andric@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/config.h>
#include <linux/device.h>
#include <asm/hardware.h>
#include <linux/serial_8250.h>
#include <linux/spi/spi.h>
#include <asm/arch/tomtomgo-map.h>
#include <asm/arch/tomtomgo-irq.h>
#include <linux/spi/spi.h>
#include <linux/spi/spi_bitbang.h>
#include <asm/arch/spi.h>
#include <asm/arch/spi-gpio.h>
#include <barcelona/gacc.h>
#include <barcelona/gyro.h>
#include <barcelona/Barc_snd.h>
#include "s3c2410-bus.h"
#include "tomtomgo-devs.h"
#include <linux/si4710.h>
#ifdef CONFIG_I2C_S3C24XX_GPIO
#include <barcelona/gopins.h>
#include <barcelona/s3c24xx_gpio_i2c.h>
#endif

/* TomTom GO IDE device */
//#if defined CONFIG_BARCELONA_IDE || defined CONFIG_BARCELONA_IDE_MODULE
static struct resource tomtomgo_ide_resource[] = {
	[0] = {
		.start = TOMTOMGO_PA_IDE,
		.end   = TOMTOMGO_PA_IDE + TOMTOMGO_SZ_IDE - 1,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = TOMTOMGO_IRQ_IDE,
		.end   = TOMTOMGO_IRQ_IDE,
		.flags = IORESOURCE_IRQ,
	},
};

struct platform_device tomtomgo_device_ide = {
	.name          = "tomtomgo-ide",
	.id            = -1,
	.num_resources = ARRAY_SIZE(tomtomgo_ide_resource),
	.resource      = tomtomgo_ide_resource,
};

EXPORT_SYMBOL(tomtomgo_device_ide);
//#endif /* CONFIG_BARCELONA_IDE || CONFIG_BARCELONA_IDE_MODULE */

/* TomTom GO hardware detection device */
#if defined CONFIG_BARCELONA_HW || defined CONFIG_BARCELONA_HW_MODULE
struct platform_device tomtomgo_device_hw = {
	.name          = "tomtomgo-hw",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_hw);
#endif /* CONFIG_BARCELONA_HW || CONFIG_BARCELONA_HW_MODULE */

/* TomTom GO ADC device */
#if defined CONFIG_BARCELONA_ADC || defined CONFIG_BARCELONA_ADC_MODULE
static struct resource tomtomgo_adc_resource[] = {
	[0] = {
		.start = S3C2410_PA_ADC,
		.end   = S3C2410_PA_ADC + S3C24XX_SZ_ADC - 1,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = IRQ_TC,
		.end   = IRQ_ADC,
		.flags = IORESOURCE_IRQ,
	},
};

/* struct adc_channels is declared in \golinux\include\barcelona\Barc_adc.h */
struct adc_channels tomtomgo_adc_channels [] =
{
	{
		.name   = "light_sensor",
		.number = ADC_LX_OUT
	},

    /* Additional ADC channels attributes may be defined here ...*/

	{
		/* This last item MUST remain at the last position! */
		.name   = NULL,
		.number = ADC_CHANNELS
	}
} ;

struct platform_device tomtomgo_device_adc = {
	.name          = "tomtomgo-adc",
	.id            = -1,
	.num_resources = ARRAY_SIZE(tomtomgo_adc_resource),
	.resource      = tomtomgo_adc_resource,
	.dev.platform_data = &tomtomgo_adc_channels
};

EXPORT_SYMBOL(tomtomgo_device_adc);
#endif /* CONFIG_BARCELONA_ADC || CONFIG_BARCELONA_ADC_MODULE */

/* TomTom GO accelerometer device */
#if defined CONFIG_BARCELONA_ACC || defined CONFIG_BARCELONA_ACC_MODULE
struct platform_device tomtomgo_device_acc = {
	.name          = "tomtomgo-acc",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_acc);
#endif /* CONFIG_BARCELONA_ACC || CONFIG_BARCELONA_ACC_MODULE */

/* TomTom GO battery device */
#if defined CONFIG_BARCELONA_BAT || defined CONFIG_BARCELONA_BAT_MODULE
struct platform_device tomtomgo_device_bat = {
	.name          = "tomtomgo-bat",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_bat);
#endif /* CONFIG_BARCELONA_BAT || CONFIG_BARCELONA_BAT_MODULE */

/* buspower driver */
#if defined CONFIG_BARCELONA_BUSPOWER || defined CONFIG_BARCELONA_BUSPOWER_MODULE
struct platform_device tomtomgo_device_buspower = {
	.name          = "tomtomgo-buspower",
	.id            = -1,
};
EXPORT_SYMBOL(tomtomgo_device_buspower);
#endif /* CONFIG_BARCELONA_BUSPOWER || CONFIG_BARCELONA_BUSPOWER_MODULE */

/* TomTom GO GPIO device */
#if defined CONFIG_BARCELONA_GPIO || defined CONFIG_BARCELONA_GPIO_MODULE
struct platform_device tomtomgo_device_gpio = {
	.name          = "tomtomgo-gpio",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_gpio);
#endif /* CONFIG_BARCELONA_GPIO || CONFIG_BARCELONA_GPIO_MODULE */

/* TomTom GO GPS device */
#if defined CONFIG_BARCELONA_GPS || defined CONFIG_BARCELONA_GPS_MODULE
static struct resource tomtomgo_gps_resource[] = {
	[0] = {
		.start = TOMTOMGO_IRQ_GPS_PPS,
		.end   = TOMTOMGO_IRQ_GPS_PPS,
		.flags = IORESOURCE_IRQ,
	},
};

struct platform_device tomtomgo_device_gps = {
	.name          = "tomtomgo-gps",
	.id            = -1,
	.num_resources = ARRAY_SIZE(tomtomgo_gps_resource),
	.resource      = tomtomgo_gps_resource,
};

EXPORT_SYMBOL(tomtomgo_device_gps);
#endif /* CONFIG_BARCELONA_GPS || CONFIG_BARCELONA_GPS_MODULE */

/* TomTom GO PWM device */
#if defined CONFIG_BARCELONA_PWM || defined CONFIG_BARCELONA_PWM_MODULE
struct platform_device tomtomgo_device_pwm = {
	.name          = "tomtomgo-pwm",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_pwm);
#endif /* CONFIG_BARCELONA_PWM || CONFIG_BARCELONA_PWM_MODULE */

/* TomTom GO S3C Backlight device */
#if defined CONFIG_BACKLIGHT_S3C || defined CONFIG_BACKLIGHT_S3C_MODULE
struct platform_device tomtomgo_device_s3c = {
	.name          = "backlight_s3c",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_s3c);
#endif /* CONFIG_BACKLIGHT_S3C || CONFIG_BACKLIGHT_S3C_MODULE */

/* TomTom GO remote control device */
#if defined CONFIG_BARCELONA_RC || defined CONFIG_BARCELONA_RC_MODULE
struct platform_device tomtomgo_device_rc = {
	.name          = "tomtomgo-rc",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_rc);
#endif /* CONFIG_BARCELONA_RC || CONFIG_BARCELONA_RC_MODULE */

/* TomTom GO touchscreen */
#if defined CONFIG_BARCELONA_TS || defined CONFIG_BARCELONA_TS_MODULE
struct platform_device tomtomgo_device_ts = {
	.name          = "tomtomgo-ts",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_ts);
#endif /* CONFIG_BARCELONA_TS || CONFIG_BARCELONA_TS_MODULE */


/* TomTom GO touchscreen input system*/
#if defined CONFIG_BARCELONA_TSINPUT || defined CONFIG_BARCELONA_TSINPUT_MODULE
struct platform_device tomtomgo_device_tsinput = {
	.name          = "tomtomgo-tsinput",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_tsinput);
#endif /* CONFIG_BARCELONA_TSINPUT || CONFIG_BARCELONA_TSINPUT_MODULE */

/* TomTom GO legacy usb mode detection driver */
#if defined CONFIG_BARCELONA_USBMODE || defined CONFIG_BARCELONA_USBMODE_MODULE
struct platform_device tomtomgo_device_usbmode = {
	.name          = "tomtomgo-usbmode",
	.id            = -1,
};
EXPORT_SYMBOL(tomtomgo_device_usbmode);
#endif /* CONFIG_BARCELONA_USBMODE || CONFIG_BARCELONA_USBMODE_MODULE */

/* TomTom GO Touchpad device */
#if defined CONFIG_TOMTOMGO_TOUCHPAD
static struct resource tomtomgo_tp_resource[] = {
	[0] = {
		.start = IRQ_EINT20,
		.end   = IRQ_EINT23,
		.flags = IORESOURCE_IRQ,
	}
};

struct platform_device tomtomgo_device_tp = {
	.name          = "synaptics-mep-bus",
	.id            = -1,
	.num_resources = ARRAY_SIZE(tomtomgo_tp_resource),
	.resource      = tomtomgo_tp_resource
};
EXPORT_SYMBOL(tomtomgo_device_tp);
#endif /* CONFIG_TOMTOMGO_TOUCHPAD */

#if defined (CONFIG_BARCELONA_GYRO_FUJITSU) || defined (CONFIG_BARCELONA_GYRO_FUJITSU_MODULE) || \
	defined(CONFIG_BARCELONA_GYRO_ING300) || defined(CONFIG_BARCELONA_GYRO_ING300_MODULE)

struct gyro_platform_data tomtomgo_gyro_info = {
  	.device_nr	= MKDEV( GYRO_MAJOR, 0),
};
EXPORT_SYMBOL(tomtomgo_gyro_info);

struct platform_device tomtomgo_device_gyro = {
	.name				= "tomtomgo-gyro",
	.id					= -1, 	
	.dev.platform_data 	= &tomtomgo_gyro_info
 };
EXPORT_SYMBOL(tomtomgo_device_gyro);
#endif /*	defined(CONFIG_BARCELONA_GYRO) || defined(CONFIG_BARCELONA_GYRO_MODULE) */

/* TomTom GO buzzer device */
#if defined CONFIG_BARCELONA_BUZ || defined CONFIG_BARCELONA_BUZ_MODULE
struct platform_device tomtomgo_device_buz = {
	.name          = "tomtomgo-buz",
	.id            = -1,
};
EXPORT_SYMBOL(tomtomgo_device_buz);
#endif /* CONFIG_BARCELONA_BUZ || CONFIG_BARCELONA_BUZ_MODULE */

#if defined CONFIG_BARCELONA_EXTERNAL_UART || defined CONFIG_BARCELONA_EXTERNAL_UART_MODULE

/**
 * Some fields in these structure are determined at runtime!
 * Provide two channels, tomtomgo_setup_dynamic_devices() will
 * terminate the list if there is only one external uart.  
 */
 
struct s3c2410_bank_settings tomtomgo_external_uart_bus_settings[] = { 
	[0] = {
		.bank_nr      = -1, 	/* Is filled in dynamically later	*/
		.bus_width    = BUS_WIDTH_16,
		.wait_status  = WAIT_DISABLE,
		.timing.t_acs = T_ACS_CLKS_4,
		.timing.t_cos = T_COS_CLKS_4,
		.timing.t_acc = T_ACC_CLKS_14,
		.timing.t_coh = T_COH_CLKS_4,
		.timing.t_cah = T_CAH_CLKS_4,
		.timing.t_acp = T_ACP_CLKS_2,
		.timing.pmc   = PMC_NORMAL,
	},
	[1] = {
		.bank_nr      = -1, 	/* Is filled in dynamically later	*/
		.bus_width    = BUS_WIDTH_16,
		.wait_status  = WAIT_DISABLE,
		.timing.t_acs = T_ACS_CLKS_4,
		.timing.t_cos = T_COS_CLKS_4,
		.timing.t_acc = T_ACC_CLKS_14,
		.timing.t_coh = T_COH_CLKS_4,
		.timing.t_cah = T_CAH_CLKS_4,
		.timing.t_acp = T_ACP_CLKS_2,
		.timing.pmc   = PMC_NORMAL,
	}
};
EXPORT_SYMBOL( tomtomgo_external_uart_bus_settings );

struct plat_serial8250_port tomtomgo_external_uart_ports[] = {
	{
		.uartclk   = 0xffffffff, /* Indicate to 8250.c that clock rate must be read from tomtom-uart.c */
		.regshift  = 1,
		.iotype    = UPIO_MEM,
		.flags     = UPF_BOOT_AUTOCONF | UPF_SKIP_TEST | UPF_IOREMAP,
	},
	{
		.uartclk   = 0xffffffff, /* Indicate to 8250.c that clock rate must be read from tomtom-uart.c */
		.regshift  = 1,
		.iotype    = UPIO_MEM,
		.flags     = UPF_BOOT_AUTOCONF | UPF_SKIP_TEST | UPF_IOREMAP,
	},
	{
		.flags     = 0, /* serves as a terminator for serial8250 driver */
	}
};
EXPORT_SYMBOL(tomtomgo_external_uart_ports);

struct platform_device tomtomgo_device_external_uart = {
  	.name              = "serial8250",
  	.id                = 0,
  	.dev.platform_data = &tomtomgo_external_uart_ports[0],
};

EXPORT_SYMBOL(tomtomgo_device_external_uart);

#endif /* CONFIG_BARCELONA_EXTERNAL_UART */


#if defined(CONFIG_SPI_MASTER)

#if defined(CONFIG_BARCELONA_GACC_SMB365) || defined(CONFIG_BARCELONA_GACC_SMB365_MODULE)
static struct gacc_platform_data tomtomgo_smb365_info = {
	.device_nr	= MKDEV( 168, 0),
};
#endif /* CONFIG_BARCELONA_GACC_SMB365_GACC	*/

#if defined(CONFIG_BARCELONA_GACC_KXP74) || defined(CONFIG_BARCELONA_GACC_KXP74_MODULE)
static struct gacc_platform_data tomtomgo_kxp74_info = {
	.device_nr	= MKDEV( 168, 0),
};
#endif /* CONFIG_BARCELONA_GACC_KXP74_GACC	*/

struct spi_board_info tomtomgo_spi_board_info[] = {
#if defined(CONFIG_BARCELONA_GACC_SMB365) || defined(CONFIG_BARCELONA_GACC_SMB365_MODULE)
	[SPIDEV_SMB365] = {	
		/* Bosch SMB365 Accelerometer	*/
		.modalias	= "smb365",
		.platform_data	= &tomtomgo_smb365_info,
		.mode		= SPI_MODE_3 | SPI_CS_HIGH,	/* Always active high, the gopin_t IO libs will handle invertedness	*/
		//.max_speed_hz	= 6000000, /* SMB365 data sheets says 8 MHz is abs. max	*/
		.max_speed_hz	= 1000000, /* DEBUG  */
		.bus_num	= 0,
		/* Note: chip_select is setup runtime	*/
	},
#endif /* CONFIG_BARCELONA_GACC_SMB365 || CONFIG_BARCELONA_GACC_SMB365_MODULE */
#if defined(CONFIG_BARCELONA_GACC_KXP74) || defined(CONFIG_BARCELONA_GACC_KXP74_MODULE)
	[SPIDEV_KXP74] = {
                /* Kionix KXP74/KXR94 Accelerometer   */
                .modalias       = "kxp74",			/* This is the default. If this is the KXR94, then we will */
								/* change this name. */
                .platform_data  = &tomtomgo_kxp74_info,
                .mode           = SPI_MODE_3 | SPI_CS_HIGH,     /* Always active high, the gopin_t IO libs will handle invertedness     */
                //.max_speed_hz = 6000000, /* SMB365 data sheets says 8 MHz is abs. max */
                .max_speed_hz   = 1000000, /* DEBUG  */
                .bus_num        = 0,
                /* Note: chip_select is setup runtime   */
        },
#endif /* defined(CONFIG_BARCELONA_GACC_KXP74) || CONFIG_BARCELONA_GACC_KXP74_MODULE	*/
};
EXPORT_SYMBOL(tomtomgo_spi_board_info);


#if defined(CONFIG_SPI_S3C24XX_GPIO) || defined(CONFIG_SPI_S3C24XX_GPIO_MODULE)
/* Additional info for spi_s3c24xxbitbang.c SPI master */

extern void tomtom_spigpio_set_cs(struct spi_device *spi, int value, int pol); /* in drivers/spi/spi_s3c24xx_gpio.c	*/

struct s3c2410_spigpio_info tomtomgo_spigpio_info = {
	/* 
	 * N.B: pin_cs is ignored because the chip select is device specific. We define tomtom_spigpio_set_cs()
	 * that will use the chip_select field from the spi device.
	 */
	.board_size	= ARRAY_SIZE(tomtomgo_spi_board_info),
	.board_info	= &tomtomgo_spi_board_info[0],
	.chip_select	= tomtom_spigpio_set_cs,
};	

EXPORT_SYMBOL(tomtomgo_spigpio_info);

struct platform_device s3c24xx_device_spigpio = {
        .name              = "s3c24xx-spi-gpio",
        .id                = -1,
        .dev.platform_data = &tomtomgo_spigpio_info,
};
EXPORT_SYMBOL(s3c24xx_device_spigpio);

#endif /* SPI_S3C24XX_GPIO || SPI_S3C24XX_GPIO_MODULE	*/

#endif /* CONFIG_SPI_MASTER	*/

struct barcelona_sound_info tomtomgo_sound_info = {
	.divider = 0,
};

EXPORT_SYMBOL( tomtomgo_sound_info );

#if defined( CONFIG_I2C )
#if defined( CONFIG_SI4710_FM ) 

static struct fm_transmitter_info tomtomgo_fmx_info = {
	.slave_address	= SI4710_I2C_SLAVE_ADDR,
	.device_nr		= MKDEV( SI4710_MAJOR, SI4710_MINOR),
};

struct platform_device tomtomgo_device_fmx = {
	.name				= SI4710_DEVNAME,
	.id					= -1,
	.dev.platform_data	= &tomtomgo_fmx_info
};

EXPORT_SYMBOL(tomtomgo_device_fmx);
#endif /* defined( CONFIG_SI4710_FM ) */

#ifdef CONFIG_I2C_S3C24XX_GPIO
static struct s3c24xx_gpio_i2c_pins s3c24xx_i2c_gpio_pins = {
	.sda_pin	= 0,
	.scl_pin	= 0,
};

struct platform_device s3c24xx_gpio_i2c_controller = {
	.name		= "S3C24XX-GPIO-I2C",
	.id		= 0,
	.dev		= {
		.platform_data = &s3c24xx_i2c_gpio_pins,
	},
        .num_resources  = 0
};
EXPORT_SYMBOL( s3c24xx_gpio_i2c_controller );
#endif /* ifdef I2C_S3C24XX_GPIO */

#ifdef CONFIG_GPRS_INTERFACE
struct platform_device tomtomgo_gprs_modem = {
	.name		= "gprs-interface",
	.id		= -1,
	.num_resources	= 0
};

EXPORT_SYMBOL( tomtomgo_gprs_modem );
#endif /* ifdef CONFIG_GPRS_INTERFACE */

#ifdef CONFIG_BARCELONA_DOCK
static struct resource s3c24xx_dock_dongle_resource[]= {
	[0] = {
		.start	=0,
		.end	=0,
		.flags	=IORESOURCE_IRQ,
	}
};

struct platform_device s3c24xx_dock_dongle = {
	.name		= "dock-dongle",
	.id		= 0,
	.num_resources	= ARRAY_SIZE( s3c24xx_dock_dongle_resource ),
	.resource	= s3c24xx_dock_dongle_resource,
};
EXPORT_SYMBOL( s3c24xx_dock_dongle );
#endif /* CONFIG_BARCELONA_DOCK */

#endif /* defined( CONFIG_I2C )	*/ 

/* EOF */
