/* drivers/video/tomtom/lms430hf12.c
 *
 * Specific functionality for the Samsung LMS430HF12 screen. See:
 * http://www.samsung.com/Products/TFTLCD/Small_n_Medium/
 *
 * Copyright (C) 2005 TomTom BV <http://www.tomtom.com/>
 * Author: Jeroen Taverne <jeroen.taverne@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifdef __KERNEL__
#include <linux/delay.h>
#include <linux/cpufreq.h>
#include <barcelona/gopins.h>
#include <linux/err.h>
#include <asm/hardware/clock.h>
#endif /* __KERNEL */
#ifdef __BOOTLOADER__
#include "timer.h"
#include "gopins.h"
#endif /* __BOOTLOADER__ */
#include "lcdregs.h"
#include "screeninfo.h"
#include "framerate.h"
#include "lms430hf12.h"

// LCDCON1
#define MMODE		0
#define PNR_MODE	3
#define BPP_MODE	12
#define ENVID		0

// LCDCON2
#define VBPD		0
#define VFPD		2
#define VSPW		10

// LCDCON3
#define HBPD		45
#define HFPD		7

// LCDCON4
#define MVAL		13
#define HSPW		41

// LCDCON5
#define BPP24BL		0
#define FRM565		1
#define INVVCLK		0
#define INVVLINE	1
#define INVVFRAME	1
#define INVVD		0
#define INVVDEN		0
#define INVPWREN	0
#define INVLEND		1
#define PWREN		1
#define ENLEND		0
#define BSWP		0
#define HWSWP		1

void LMS430HF12_Init(void)
{
	unsigned long int	divider=(unsigned long int) get_clkval( &ttgfb_lms430hf12_lcdspecs,
									ttgfb_lms430hf12_lcdspecs.refresh_max,
									ttgfb_lms430hf12_lcdspecs.curr_hclk );
	unsigned long int	hozval;
	unsigned long int	lineval;

	/* Get HOZVAL and LINEVAL. */
	get_hozlineval( &ttgfb_lms430hf12_lcdspecs, &hozval, &lineval );

	/* Start display controller */
	rLCDCON1 &= ~1;
	rTCONSEL = 0;
	rLCDCON1 = (divider << 8) | (MMODE << 7) | (((unsigned long int) ttgfb_lms430hf12_lcdspecs.pnr_mode) << 5) |
						   (((unsigned long int) ttgfb_lms430hf12_lcdspecs.bpp_mode) << 1) | (ENVID << 0);
	rLCDCON2 = (((unsigned long int) ttgfb_lms430hf12_lcdspecs.screen.tft.vbpd) << 24) |
		   (lineval << 14) |
		   (((unsigned long int) ttgfb_lms430hf12_lcdspecs.screen.tft.vfpd) << 6) |
		   (((unsigned long int) ttgfb_lms430hf12_lcdspecs.screen.tft.vspw) << 0);
	rLCDCON3 = (((unsigned long int) ttgfb_lms430hf12_lcdspecs.screen.tft.hbpd) << 19) |
		   (hozval << 8) |
		   (((unsigned long int) ttgfb_lms430hf12_lcdspecs.screen.tft.hfpd) << 0);
	rLCDCON4 = (MVAL << 8) | (((unsigned long int) ttgfb_lms430hf12_lcdspecs.screen.tft.hspw) << 0);
	rLCDCON5 = (BPP24BL << 12) | (FRM565 << 11) | (INVVCLK << 10) | (INVVLINE << 9) | (INVVFRAME << 8) | (INVVD << 7) |
		   (INVVDEN << 6) | (INVPWREN << 5) | (INVLEND << 4) | (PWREN << 3) | (ENLEND << 2) | (BSWP << 1) | (HWSWP << 0);

	// Start display controller
	rLCDCON1 |= 1;

  mdelay(12); //T2: 10msec
  
  IO_Deactivate(LCD_CS);//this is actually PON pin for this LCM
  mdelay(200); // T3:wait for at least 10 frames before power on backlight
}

void LMS430HF12_Off(void)
{
	mdelay(200); // T3:wait for at least 10 frames after power off backlight
	IO_Activate(LCD_CS);//this is actually PON pin for this LCM
	mdelay(120); // T5:at lease 5 frames+T4
	// Stop display controller
	rLCDCON1 = 0;
	mdelay(12);//T4: at lease 10msec
}

#ifdef CONFIG_CPU_FREQ
struct lcd_screen_info ttgfb_lms430hf12_lcdspecs = {"SAM", "LMS430HF12", 95, 54, 480, 272, 25, 75, LMS430HF12_Init, LMS430HF12_Off, NULL,
						  PNR_TFT, BPP_16BPP_TFT, 0, {{VSPW, VBPD, VFPD, HSPW, HBPD, HFPD}},
						  lcd_def_freq_policy, lcd_def_freq_trans};
#else
struct lcd_screen_info ttgfb_lms430hf12_lcdspecs = {"SAM", "LMS430HF12", 95, 54, 480, 272, 25, 75, LMS430HF12_Init, LMS430HF12_Off, NULL,
						  PNR_TFT, BPP_16BPP_TFT, 0, {{VSPW, VBPD, VFPD, HSPW, HBPD, HFPD}}};
#endif
/* EOF */
