/* drivers/video/tomtom/lq043t3dw01.c
 *
 * Specific functionality for the Sharp LQ043T3DW01 screen. See:
 * http://www.samsung.com/Products/TFTLCD/Small_n_Medium/
 *
 * Copyright (C) 2005 TomTom BV <http://www.tomtom.com/>
 * Author: Jeroen Taverne <jeroen.taverne@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifdef __KERNEL__
#include <linux/delay.h>
#include <linux/cpufreq.h>
#include <barcelona/gopins.h>
#include <linux/err.h>
#include <asm/hardware/clock.h>
#endif /* __KERNEL */
#ifdef __BOOTLOADER__
#include "timer.h"
#include "gopins.h"
#endif /* __BOOTLOADER__ */
#include "lcdregs.h"
#include "screeninfo.h"
#include "framerate.h"
#include "lq043t3dw01.h"

// LCDCON1
#define MMODE		0
#define PNR_MODE	3
#define BPP_MODE	12
#define ENVID		0

// LCDCON2
#define VBPD		((2-1)&0x1ff)
#define VFPD		((2-1)&0xff)
#define VSPW		((2-1)&0x3f)

// LCDCON3
#define HBPD		((3-1)&0x7f)
#define HFPD		((2-1)&0xff)

// LCDCON4
#define MVAL		13
#define HSPW		((13-1)&0xff)

// LCDCON5
#define BPP24BL		0
#define FRM565		1
#define INVVCLK		1
#define INVVLINE	1
#define INVVFRAME	1
#define INVVD		0
#define INVVDEN		0
#define INVPWREN	0
#define INVLEND		1
#define PWREN		1
#define ENLEND		0
#define BSWP		0
#define HWSWP		1

static void delay_1us_more(void)
{
#ifdef __KERNEL__
    udelay(1);
#else
    mdelay(1);
#endif	
}

void LQ043T3DW01_Init(void)
{
	unsigned long int	divider=(unsigned long int) get_clkval( &ttgfb_lq043t3dw01_lcdspecs,
									ttgfb_lq043t3dw01_lcdspecs.refresh_max,
									ttgfb_lq043t3dw01_lcdspecs.curr_hclk );
	unsigned long int	hozval;
	unsigned long int	lineval;
  unsigned int vspw_cpu,hspw_cpu, hbpd_cpu, vbpd_cpu, lcm_hbpd, lcm_vbpd;
  
	/* Get HOZVAL and LINEVAL. */
	get_hozlineval( &ttgfb_lq043t3dw01_lcdspecs, &hozval, &lineval );

	vspw_cpu = ttgfb_lq043t3dw01_lcdspecs.screen.tft.vspw+1;//2
	vbpd_cpu = ttgfb_lq043t3dw01_lcdspecs.screen.tft.vbpd+1;//1	
	hspw_cpu = ttgfb_lq043t3dw01_lcdspecs.screen.tft.hspw+1;//13
	hbpd_cpu = ttgfb_lq043t3dw01_lcdspecs.screen.tft.hbpd+1;//3
	lcm_hbpd = hbpd_cpu+hspw_cpu; //should be 16
	lcm_vbpd = vbpd_cpu+vspw_cpu; //should be 3
	
	/* Start display controller */
	rLCDCON1 &= ~1;
	rTCONSEL = 0;
	rLCDCON1 = (divider << 8) | (MMODE << 7) | (((unsigned long int) ttgfb_lq043t3dw01_lcdspecs.pnr_mode) << 5) |
						   (((unsigned long int) ttgfb_lq043t3dw01_lcdspecs.bpp_mode) << 1) | (ENVID << 0);
	rLCDCON2 = (((unsigned long int) ttgfb_lq043t3dw01_lcdspecs.screen.tft.vbpd) << 24) |
		   (lineval << 14) |
		   (((unsigned long int) ttgfb_lq043t3dw01_lcdspecs.screen.tft.vfpd) << 6) |
		   (((unsigned long int) ttgfb_lq043t3dw01_lcdspecs.screen.tft.vspw) << 0);
	rLCDCON3 = (((unsigned long int) ttgfb_lq043t3dw01_lcdspecs.screen.tft.hbpd) << 19) |
		   (hozval << 8) |
		   (((unsigned long int) ttgfb_lq043t3dw01_lcdspecs.screen.tft.hfpd) << 0);
	rLCDCON4 = (MVAL << 8) | (((unsigned long int) ttgfb_lq043t3dw01_lcdspecs.screen.tft.hspw) << 0);
	rLCDCON5 = (BPP24BL << 12) | (FRM565 << 11) | (INVVCLK << 10) | (INVVLINE << 9) | (INVVFRAME << 8) | (INVVD << 7) |
		   (INVVDEN << 6) | (INVPWREN << 5) | (INVLEND << 4) | (PWREN << 3) | (ENLEND << 2) | (BSWP << 1) | (HWSWP << 0);

	// Start display controller
	IO_Deactivate(VDEN); // do not use VDEN
	mdelay(1);
	IO_Deactivate(LCD_CS); //SHUT as VCI
  delay_1us_more();
  	
	rLCDCON1 |= 1;

  delay_1us_more();
  
  IO_Activate(LCD_CS); //SHUT: VCI->GND
  mdelay(300); // at least wait for 15 frames
  
}

void LQ043T3DW01_Off(void)
{
	IO_Deactivate(LCD_CS); //SHUT : GND-> VCI
	mdelay(60); // at least wait for 3 frames

	// Stop display controller
	rLCDCON1 = 0;

}

#ifdef CONFIG_CPU_FREQ
struct lcd_screen_info ttgfb_lq043t3dw01_lcdspecs = {"SHP", "LQ043T3DW01", 95, 54, 480, 272, 25, 62, LQ043T3DW01_Init, LQ043T3DW01_Off, NULL,
						  PNR_TFT, BPP_16BPP_TFT, 0, {{VSPW, VBPD, VFPD, HSPW, HBPD, HFPD}},
						  lcd_def_freq_policy, lcd_def_freq_trans};
#else
struct lcd_screen_info ttgfb_lq043t3dw01_lcdspecs = {"SHP", "LQ043T3DW01", 95, 54, 480, 272, 25, 62, LQ043T3DW01_Init, LQ043T3DW01_Off, NULL,
						  PNR_TFT, BPP_16BPP_TFT, 0, {{VSPW, VBPD, VFPD, HSPW, HBPD, HFPD}}};
#endif
/* EOF */
